@@ -17,7 +17,7 @@
 %                               February 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2020 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -39,31 +39,30 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/annotate.h"
-#include "magick/artifact.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/composite-private.h"
-#include "magick/draw.h"
-#include "magick/draw-private.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/option.h"
-#include "magick/pixel-accessor.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/resource_.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/string-private.h"
-#include "magick/utility.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/annotate.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/composite-private.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/draw-private.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/option.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/resource_.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/string-private.h"
+#include "MagickCore/utility.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -97,8 +96,7 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
 {
   char
     *caption,
-    geometry[MaxTextExtent],
-    lines[MaxTextExtent],
+    geometry[MagickPathExtent],
     *text;
 
   const char
@@ -135,24 +133,27 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
   assert(exception->signature == MagickCoreSignature);
-  image=AcquireImage(image_info);
+  image=AcquireImage(image_info,exception);
   (void) ResetImagePage(image,"0x0+0+0");
   if ((image->columns != 0) && (image->rows != 0))
-    (void) SetImageBackgroundColor(image);
+    (void) SetImageBackgroundColor(image,exception);
   /*
     Format caption.
   */
   option=GetImageOption(image_info,"filename");
   if (option == (const char *) NULL)
-    caption=InterpretImageProperties(image_info,image,image_info->filename);
+    caption=InterpretImageProperties((ImageInfo *) image_info,image,
+      image_info->filename,exception);
   else
     if (LocaleNCompare(option,"caption:",8) == 0)
-      caption=InterpretImageProperties(image_info,image,option+8);
+      caption=InterpretImageProperties((ImageInfo *) image_info,image,option+8,
+        exception);
     else
-      caption=InterpretImageProperties(image_info,image,option);
+      caption=InterpretImageProperties((ImageInfo *) image_info,image,option,
+        exception);
   if (caption == (char *) NULL)
     return(DestroyImageList(image));
-  (void) SetImageProperty(image,"caption",caption);
+  (void) SetImageProperty(image,"caption",caption,exception);
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
   width=(size_t) floor(draw_info->pointsize*strlen(caption)+0.5);
   if (AcquireMagickResource(WidthResource,width) == MagickFalse)
@@ -172,45 +173,43 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   if (image->columns == 0)
     {
       text=AcquireString(caption);
-      i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+      i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
       (void) CloneString(&draw_info->text,text);
       text=DestroyString(text);
-      (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
+      (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
         -metrics.bounds.x1,metrics.ascent);
       if (draw_info->gravity == UndefinedGravity)
         (void) CloneString(&draw_info->geometry,geometry);
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
       image->columns=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
     }
   if (image->rows == 0)
     {
       split=MagickTrue;
       text=AcquireString(caption);
-      i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+      i=FormatMagickCaption(image,draw_info,split,&metrics,&text,exception);
       (void) CloneString(&draw_info->text,text);
       text=DestroyString(text);
-      (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
+      (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
         -metrics.bounds.x1,metrics.ascent);
       if (draw_info->gravity == UndefinedGravity)
         (void) CloneString(&draw_info->geometry,geometry);
-      status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+      status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
       image->rows=(size_t) ((i+1)*(metrics.ascent-metrics.descent+
         draw_info->interline_spacing+draw_info->stroke_width)+0.5);
     }
   if (status != MagickFalse)
-    status=SetImageExtent(image,image->columns,image->rows);
+    status=SetImageExtent(image,image->columns,image->rows,exception);
   if (status == MagickFalse)
     { 
       caption=DestroyString(caption);
       draw_info=DestroyDrawInfo(draw_info);
-      InheritException(exception,&image->exception);
       return(DestroyImageList(image));
     }
-  if (SetImageBackgroundColor(image) == MagickFalse)
+  if (SetImageBackgroundColor(image,exception) == MagickFalse)
     {
       caption=DestroyString(caption);
       draw_info=DestroyDrawInfo(draw_info);
-      InheritException(exception,&image->exception);
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
@@ -226,50 +225,61 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
       /*
         Auto fit text into bounding box.
       */
-      for (n=0; n < 32; n++, draw_info->pointsize*=2.0)
-      {
-        text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
-        (void) CloneString(&draw_info->text,text);
-        text=DestroyString(text);
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
-          -metrics.bounds.x1,metrics.ascent);
-        if (draw_info->gravity == UndefinedGravity)
-          (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
-        if (status == MagickFalse)
-          break;
-        width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-        height=(size_t) floor(metrics.height+draw_info->interline_spacing+
-          draw_info->stroke_width+0.5);
-        if ((image->columns != 0) && (image->rows != 0))
+      option=GetImageOption(image_info,"caption:max-pointsize");
+      if (option != (const char*) NULL)
+        {
+          high=StringToDouble(option,(char**) NULL);
+          if (high < 1.0)
+            high=1.0;
+          high+=1.0;
+        }
+      else
+        {
+          for (n=0; n < 32; n++, draw_info->pointsize*=2.0)
           {
-            if ((width >= image->columns) && (height >= image->rows))
+            text=AcquireString(caption);
+            i=FormatMagickCaption(image,draw_info,split,&metrics,&text,
+              exception);
+            (void) CloneString(&draw_info->text,text);
+            text=DestroyString(text);
+            (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
+              -metrics.bounds.x1,metrics.ascent);
+            if (draw_info->gravity == UndefinedGravity)
+              (void) CloneString(&draw_info->geometry,geometry);
+            status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
+            if (status == MagickFalse)
               break;
+            width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
+            height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
+            if ((image->columns != 0) && (image->rows != 0))
+              {
+                if ((width >= image->columns) && (height >= image->rows))
+                  break;
+              }
+            else
+              if (((image->columns != 0) && (width >= image->columns)) ||
+                  ((image->rows != 0) && (height >= image->rows)))
+                break;
           }
-        else
-          if (((image->columns != 0) && (width >= image->columns)) ||
-              ((image->rows != 0) && (height >= image->rows)))
-            break;
-      }
-      high=draw_info->pointsize;
+          high=draw_info->pointsize;
+        }
       for (low=1.0; (high-low) > 0.5; )
       {
         draw_info->pointsize=(low+high)/2.0;
         text=AcquireString(caption);
-        i=FormatMagickCaption(image,draw_info,split,&metrics,&text);
+        i=FormatMagickCaption(image,draw_info,split,&metrics,&text,
+          exception);
         (void) CloneString(&draw_info->text,text);
         text=DestroyString(text);
-        (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",
+        (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",
           -metrics.bounds.x1,metrics.ascent);
         if (draw_info->gravity == UndefinedGravity)
           (void) CloneString(&draw_info->geometry,geometry);
-        status=GetMultilineTypeMetrics(image,draw_info,&metrics);
+        status=GetMultilineTypeMetrics(image,draw_info,&metrics,exception);
         if (status == MagickFalse)
           break;
         width=(size_t) floor(metrics.width+draw_info->stroke_width+0.5);
-        height=(size_t) floor(metrics.height+draw_info->interline_spacing+
-          draw_info->stroke_width+0.5);
+        height=(size_t) floor(metrics.height+draw_info->stroke_width+0.5);
         if ((image->columns != 0) && (image->rows != 0))
           {
             if ((width < image->columns) && (height < image->rows))
@@ -289,26 +299,24 @@ static Image *ReadCAPTIONImage(const ImageInfo *image_info,
   /*
     Draw caption.
   */
-  i=FormatMagickCaption(image,draw_info,split,&metrics,&caption);
+  i=FormatMagickCaption(image,draw_info,split,&metrics,&caption,exception);
   (void) CloneString(&draw_info->text,caption);
   caption=DestroyString(caption);
-  (void) FormatLocaleString(geometry,MaxTextExtent,"%+g%+g",MagickMax(
+  (void) FormatLocaleString(geometry,MagickPathExtent,"%+g%+g",MagickMax(
     draw_info->direction == RightToLeftDirection ? (double) image->columns-
-    metrics.bounds.x2 : -metrics.bounds.x1,0.0),(draw_info->gravity ==
-    UndefinedGravity ? MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0));
+    metrics.bounds.x2 : -metrics.bounds.x1,0.0),draw_info->gravity ==
+    UndefinedGravity ? MagickMax(metrics.ascent,metrics.bounds.y2) : 0.0);
   (void) CloneString(&draw_info->geometry,geometry);
-  status=AnnotateImage(image,draw_info);
+  status=AnnotateImage(image,draw_info,exception);
   if (image_info->pointsize == 0.0)
     { 
       char
-        pointsize[MaxTextExtent];
+        pointsize[MagickPathExtent];
       
-      (void) FormatLocaleString(pointsize,MaxTextExtent,"%.20g",
+      (void) FormatLocaleString(pointsize,MagickPathExtent,"%.20g",
         draw_info->pointsize);
-      (void) SetImageProperty(image,"caption:pointsize",pointsize);
+      (void) SetImageProperty(image,"caption:pointsize",pointsize,exception);
     }
-  (void) FormatLocaleString(lines,MaxTextExtent,"%.20g",(double) (i+1));
-  (void) SetImageProperty(image,"caption:lines",lines);
   draw_info=DestroyDrawInfo(draw_info);
   if (status == MagickFalse)
     {
@@ -346,11 +354,9 @@ ModuleExport size_t RegisterCAPTIONImage(void)
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("CAPTION");
+  entry=AcquireMagickInfo("CAPTION","CAPTION","Caption");
   entry->decoder=(DecodeImageHandler *) ReadCAPTIONImage;
-  entry->description=ConstantString("Caption");
-  entry->adjoin=MagickFalse;
-  entry->magick_module=ConstantString("CAPTION");
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
