@@ -13,17 +13,17 @@
 %                      Read Alias/Wavefront Image Format                      %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2019 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://imagemagick.org/script/license.php                               %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,24 +39,25 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/property.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/cache.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/property.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -112,25 +113,25 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       revision;
 
     char
-      gamma[16],
-      red_primary[24],
-      green_primary[24],
-      blue_primary[24],
-      white_point[24];
+      gamma[16+1],
+      red_primary[24+1],
+      green_primary[24+1],
+      blue_primary[24+1],
+      white_point[24+1];
 
-    long
+    int
       job_number;
 
     char
-      name[128],
-      description[128],
-      program[64],
-      machine[32],
-      user[32],
-      date[20],
-      aspect[24],
-      aspect_ratio[8],
-      chan[32];
+      name[128+1],
+      description[128+1],
+      program[64+1],
+      machine[32+1],
+      user[32+1],
+      date[20+1],
+      aspect[24+1],
+      aspect_ratio[8+1],
+      chan[32+1];
 
     short
       field;
@@ -147,10 +148,10 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
       auxiliary_bits;
 
     char
-      auxiliary[32],
-      space[36];
+      auxiliary[32+1],
+      space[36+1];
 
-    long
+    int
       next;
   } RLAInfo;
 
@@ -162,27 +163,23 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     length,
     runlength;
 
-  long
-    y;
-
-  long
-    *scanlines;
-
   MagickBooleanType
     status;
 
   MagickOffsetType
-    offset;
+    offset,
+    *scanlines;
 
-  register long
+  register ssize_t
     i,
     x;
 
-  register PixelPacket
+  register Quantum
     *q;
 
   ssize_t
-    count;
+    count,
+    y;
 
   RLAInfo
     rla_info;
@@ -194,19 +191,20 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
+  (void) memset(&rla_info,0,sizeof(rla_info));
   rla_info.window.left=(short) ReadBlobMSBShort(image);
   rla_info.window.right=(short) ReadBlobMSBShort(image);
   rla_info.window.bottom=(short) ReadBlobMSBShort(image);
@@ -218,78 +216,95 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
   rla_info.frame=(short) ReadBlobMSBShort(image);
   rla_info.storage_type=(short) ReadBlobMSBShort(image);
   rla_info.number_channels=(short) ReadBlobMSBShort(image);
+  if (rla_info.number_channels < 0)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   rla_info.number_matte_channels=(short) ReadBlobMSBShort(image);
+  if (rla_info.number_matte_channels < 0)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+  if ((rla_info.number_channels > 3) || (rla_info.number_matte_channels > 3))
+    ThrowReaderException(CoderError,"Unsupported number of channels");
   if (rla_info.number_channels == 0)
     rla_info.number_channels=3;
   rla_info.number_channels+=rla_info.number_matte_channels;
   rla_info.number_auxiliary_channels=(short) ReadBlobMSBShort(image);
   rla_info.revision=(short) ReadBlobMSBShort(image);
-  count=ReadBlob(image,16,(unsigned char *) rla_info.gamma);
-  count=ReadBlob(image,24,(unsigned char *) rla_info.red_primary);
-  count=ReadBlob(image,24,(unsigned char *) rla_info.green_primary);
-  count=ReadBlob(image,24,(unsigned char *) rla_info.blue_primary);
-  count=ReadBlob(image,24,(unsigned char *) rla_info.white_point);
-  rla_info.job_number=(long) ReadBlobMSBLong(image);
-  count=ReadBlob(image,128,(unsigned char *) rla_info.name);
-  count=ReadBlob(image,128,(unsigned char *) rla_info.description);
-  count=ReadBlob(image,64,(unsigned char *) rla_info.program);
-  count=ReadBlob(image,32,(unsigned char *) rla_info.machine);
-  count=ReadBlob(image,32,(unsigned char *) rla_info.user);
-  count=ReadBlob(image,20,(unsigned char *) rla_info.date);
-  count=ReadBlob(image,24,(unsigned char *) rla_info.aspect);
-  count=ReadBlob(image,8,(unsigned char *) rla_info.aspect_ratio);
-  count=ReadBlob(image,32,(unsigned char *) rla_info.chan);
+  (void) ReadBlob(image,16,(unsigned char *) rla_info.gamma);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.red_primary);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.green_primary);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.blue_primary);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.white_point);
+  rla_info.job_number=ReadBlobMSBSignedLong(image);
+  (void) ReadBlob(image,128,(unsigned char *) rla_info.name);
+  (void) ReadBlob(image,128,(unsigned char *) rla_info.description);
+  rla_info.description[127]='\0';
+  (void) ReadBlob(image,64,(unsigned char *) rla_info.program);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.machine);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.user);
+  (void) ReadBlob(image,20,(unsigned char *) rla_info.date);
+  (void) ReadBlob(image,24,(unsigned char *) rla_info.aspect);
+  (void) ReadBlob(image,8,(unsigned char *) rla_info.aspect_ratio);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.chan);
   rla_info.field=(short) ReadBlobMSBShort(image);
-  count=ReadBlob(image,12,(unsigned char *) rla_info.time);
-  count=ReadBlob(image,32,(unsigned char *) rla_info.filter);
+  (void) ReadBlob(image,12,(unsigned char *) rla_info.time);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.filter);
   rla_info.bits_per_channel=(short) ReadBlobMSBShort(image);
   rla_info.matte_type=(short) ReadBlobMSBShort(image);
   rla_info.matte_bits=(short) ReadBlobMSBShort(image);
   rla_info.auxiliary_type=(short) ReadBlobMSBShort(image);
   rla_info.auxiliary_bits=(short) ReadBlobMSBShort(image);
-  count=ReadBlob(image,32,(unsigned char *) rla_info.auxiliary);
+  (void) ReadBlob(image,32,(unsigned char *) rla_info.auxiliary);
   count=ReadBlob(image,36,(unsigned char *) rla_info.space);
   if ((size_t) count != 36)
     ThrowReaderException(CorruptImageError,"UnableToReadImageData");
-  rla_info.next=(long) ReadBlobMSBLong(image);
+  rla_info.next=ReadBlobMSBSignedLong(image);
   /*
     Initialize image structure.
   */
-  image->matte=rla_info.number_matte_channels != 0 ? MagickTrue : MagickFalse;
-  image->columns=1UL*rla_info.active_window.right-rla_info.active_window.left+1;
-  image->rows=1UL*rla_info.active_window.top-rla_info.active_window.bottom+1;
+  image->alpha_trait=rla_info.number_matte_channels != 0 ? BlendPixelTrait : 
+    UndefinedPixelTrait;
+  image->columns=(size_t) (rla_info.active_window.right-
+    rla_info.active_window.left+1);
+  image->rows=(size_t) (rla_info.active_window.top-
+    rla_info.active_window.bottom+1);
   if (image_info->ping != MagickFalse)
     {
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  if (SetImageExtent(image,0,0) == MagickFalse)
-    {
-      InheritException(exception,&image->exception);
-      return(DestroyImageList(image));
-    }
-  scanlines=(long *) AcquireQuantumMemory(image->rows,sizeof(*scanlines));
-  if (scanlines == (long *) NULL)
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
+  scanlines=(MagickOffsetType *) AcquireQuantumMemory(image->rows,
+    sizeof(*scanlines));
+  if (scanlines == (MagickOffsetType *) NULL)
     ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
   if (*rla_info.description != '\0')
-    (void) SetImageProperty(image,"comment",rla_info.description);
+    (void) SetImageProperty(image,"comment",rla_info.description,exception);
   /*
     Read offsets to each scanline data.
   */
-  for (i=0; i < (long) image->rows; i++)
-    scanlines[i]=(long) ReadBlobMSBLong(image);
+  for (i=0; i < (ssize_t) image->rows; i++)
+    scanlines[i]=(MagickOffsetType) ReadBlobMSBSignedLong(image);
+  if (EOFBlob(image) != MagickFalse)
+    {
+      scanlines=(MagickOffsetType *) RelinquishMagickMemory(scanlines);
+      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+    }
   /*
     Read image data.
   */
-  x=0;
-  for (y=0; y < (long) image->rows; y++)
+  for (y=0; y < (ssize_t) image->rows; y++)
   {
     offset=SeekBlob(image,scanlines[image->rows-y-1],SEEK_SET);
     if (offset < 0)
-      ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      {
+        scanlines=(MagickOffsetType *) RelinquishMagickMemory(scanlines);
+        ThrowReaderException(CorruptImageError,"ImproperImageHeader");
+      }
+    x=0;
     for (channel=0; channel < (int) rla_info.number_channels; channel++)
     {
-      length=(int) ReadBlobMSBShort(image);
+      length=ReadBlobMSBSignedShort(image);
       while (length > 0)
       {
         byte=(unsigned char) ReadBlobByte(image);
@@ -303,9 +318,9 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
           {
             while (runlength < 0)
             {
-              q=GetAuthenticPixels(image,(long) (x % image->columns),
-                (long) (y % image->rows),1,1,exception);
-              if (q == (PixelPacket *) NULL)
+              q=GetAuthenticPixels(image,(ssize_t) (x % image->columns),y,1,1,
+                exception);
+              if (q == (Quantum *) NULL)
                 break;
               byte=(unsigned char) ReadBlobByte(image);
               length--;
@@ -313,23 +328,23 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
               {
                 case 0:
                 {
-                  q->red=ScaleCharToQuantum(byte);
+                  SetPixelRed(image,ScaleCharToQuantum(byte),q);
                   break;
                 }
                 case 1:
                 {
-                  q->green=ScaleCharToQuantum(byte);
+                  SetPixelGreen(image,ScaleCharToQuantum(byte),q);
                   break;
                 }
                 case 2:
                 {
-                  q->blue=ScaleCharToQuantum(byte);
+                  SetPixelBlue(image,ScaleCharToQuantum(byte),q);
                   break;
                 }
                 case 3:
                 default:
                 {
-                  q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(byte));
+                  SetPixelAlpha(image,ScaleCharToQuantum(byte),q);
                   break;
                 }
               }
@@ -345,31 +360,31 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         runlength++;
         do
         {
-          q=GetAuthenticPixels(image,(long) (x % image->columns),
-            (long) (y % image->rows),1,1,exception);
-          if (q == (PixelPacket *) NULL)
+          q=GetAuthenticPixels(image,(ssize_t) (x % image->columns),y,1,1,
+            exception);
+          if (q == (Quantum *) NULL)
             break;
           switch (channel)
           {
             case 0:
             {
-              q->red=ScaleCharToQuantum(byte);
+              SetPixelRed(image,ScaleCharToQuantum(byte),q);
               break;
             }
             case 1:
             {
-              q->green=ScaleCharToQuantum(byte);
+              SetPixelGreen(image,ScaleCharToQuantum(byte),q);
               break;
             }
             case 2:
             {
-              q->blue=ScaleCharToQuantum(byte);
+              SetPixelBlue(image,ScaleCharToQuantum(byte),q);
               break;
             }
             case 3:
             default:
             {
-              q->opacity=(Quantum) (QuantumRange-ScaleCharToQuantum(byte));
+              SetPixelAlpha(image,ScaleCharToQuantum(byte),q);
               break;
             }
           }
@@ -381,10 +396,19 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
         while (runlength > 0);
       }
     }
-    status=SetImageProgress(image,LoadImageTag,y,image->rows);
+    if ((x/(ssize_t) rla_info.number_channels) > (ssize_t) image->columns)
+      {
+        scanlines=(MagickOffsetType *) RelinquishMagickMemory(scanlines);
+        ThrowReaderException(CorruptImageError,"CorruptImage");
+      }
+    if (EOFBlob(image) != MagickFalse)
+      break;
+    status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+      image->rows);
     if (status == MagickFalse)
       break;
   }
+  scanlines=(MagickOffsetType *) RelinquishMagickMemory(scanlines);
   if (EOFBlob(image) != MagickFalse)
     ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
       image->filename);
@@ -412,19 +436,18 @@ static Image *ReadRLAImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterRLAImage method is:
 %
-%      unsigned long RegisterRLAImage(void)
+%      size_t RegisterRLAImage(void)
 %
 */
-ModuleExport unsigned long RegisterRLAImage(void)
+ModuleExport size_t RegisterRLAImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("RLA");
+  entry=AcquireMagickInfo("RLA","RLA","Alias/Wavefront image");
   entry->decoder=(DecodeImageHandler *) ReadRLAImage;
-  entry->adjoin=MagickFalse;
-  entry->description=ConstantString("Alias/Wavefront image");
-  entry->module=ConstantString("RLA");
+  entry->flags^=CoderAdjoinFlag;
+  entry->flags|=CoderDecoderSeekableStreamFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
